"""Script for running a dam break simulation of UQ's dam break tank.


Ole Nielsen and Duncan Gray, GA - 2006 
"""


#----------------------------------------------------------------------------
# Import necessary modules
#----------------------------------------------------------------------------

# Standard modules
import time
import sys
from shutil import copy
from os import path

# Related major packages
from anuga.pyvolution.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from anuga.pyvolution.util import Screen_Catcher
from anuga.pyvolution.region import Set_region 
from anuga.fit_interpolate.interpolate import interpolate_sww2csv
import create_mesh

# Application specific imports
import project                 # Definition of file names and polygons


def main():
    #-------------------------------------------------------------------------
    # Setup archiving of simulations
    #-------------------------------------------------------------------------

    copy (project.codedirname, project.outputtimedir + 'project.py')
    copy (project.codedir + 'run_dam.py', project.outputtimedir + 'run_dam.py')
    copy (project.codedir + 'create_mesh.py',
          project.outputtimedir + 'create_mesh.py')
    print'output dir', project.outputtimedir

    #FIXME this isn't working
    #normal screen output is stored in 
    screen_output_name = project.outputtimedir + "screen_output.txt"
    screen_error_name = project.outputtimedir + "screen_error.txt"

    #-------------------------------------------------------------------------
    # Create the triangular mesh
    #-------------------------------------------------------------------------

    gate_position = 12.0
    create_mesh.generate(project.mesh_filename,
                         gate_position,
                         #is_course=True) # this creates the mesh
                         is_course=False) # this creates the mesh

    head,tail = path.split(project.mesh_filename)
    copy (project.mesh_filename,
          project.outputtimedir + tail )
    #-------------------------------------------------------------------------
    # Setup computational domain
    #-------------------------------------------------------------------------
    domain = Domain(project.mesh_filename, use_cache = False, verbose = True)
   

    print 'Number of triangles = ', len(domain)
    print 'The extent is ', domain.get_extent()
    print domain.statistics()

    
    domain.set_name(project.basename)
    domain.set_datadir(project.outputtimedir)
    domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
    domain.set_minimum_sww_depth(0.01)
    domain.set_store_vertices_uniquely(True)  # for writting to sww

    #-------------------------------------------------------------------------
    # Setup initial conditions
    #-------------------------------------------------------------------------

    slope = 0.0

    def elevation_tilt(x, y):
        return x*slope
        
    domain.set_quantity('stage', 0.06)
    domain.set_quantity('friction', 0.01) 
    domain.set_quantity('elevation',elevation_tilt)


    domain.set_quantity('elevation', filename = 'elevation.xya',
                        alpha = 10.0,verbose = True, use_cache = True)
    
    print 'Available boundary tags', domain.get_boundary_tags()
    # dam depth is set
    domain.set_region('dam','stage',0.6,
                                 location = 'unique vertices') 

    Br = Reflective_boundary(domain)
    Bd = Dirichlet_boundary([0,0,0])  # to drain the water out.
    bore = Dirichlet_boundary([1.,0.01,0])
    domain.set_boundary( {'wall': Br, 'edge': Bd, 'back':Br} )

    #-------------------------------------------------------------------------
    # Evolve system through time
    #-------------------------------------------------------------------------
    t0 = time.time()

    for t in domain.evolve(yieldstep = 0.1, finaltime = 11): #enter timestep and final time
        domain.write_time()
    
        print 'That took %.2f seconds' %(time.time()-t0)
        print 'finished'

    points = [[3.8,0.225],  #-1.8m from SWL
              [6.1,0.225],  #0.5m from SWL
              [7.6,0.225],  #2m from SWL
              [7.95,0.255], #2.35m from SWL
              [8.6,0.255],  #3m from SWL
              [9.1,0.255],  #3.5m from SWL
              [10.1,0.255]  #4.5m from SWL
              ]

    #-------------------------------------------------------------------------
    # Calculate gauge info
    #-------------------------------------------------------------------------
    interpolate_sww2csv(project.outputtimedir + project.basename \
                   +".sww",
                        points,
                        project.depth_filename + '.csv',
                        project.velocity_x_filename + '.csv',
                        project.velocity_y_filename + '.csv')

#-------------------------------------------------------------
if __name__ == "__main__":
    main()
    
