"""Script for running a dam break simulation of UQ's dam break tank.


Ole Nielsen and Duncan Gray, GA - 2006 


Issues
If running this is hand-set-up parallel, the python files are overwritten.

"""


#----------------------------------------------------------------------------
# Import necessary modules
#----------------------------------------------------------------------------

# Standard modules
import time
from time import localtime, strftime
import sys
from shutil import copy
from os import path, sep
from os.path import dirname  #, basename
from math import tan, radians

# Related major packages
from anuga.shallow_water import Domain, Reflective_boundary, \
                            Dirichlet_boundary, Time_boundary, File_boundary
from anuga.abstract_2d_finite_volumes.region import Set_region 
from anuga.fit_interpolate.interpolate import interpolate_sww2csv
from anuga.abstract_2d_finite_volumes.util import start_screen_catcher, \
     copy_code_files


# Scenario specific imports
import project                 # Definition of file names and polygons
import create_mesh



def main(friciton,beach_angle, toe_to_nails_distance,
         outputdir_name=None, is_trial_run=False):
    """
    beach angle is the angle in degrees of the sloped section
    toe_to_nails_distance is the distance from the beginning if the sloped
    section to the beginning of the nails section
    """
    basename = 'zz' + str(friction)
    if is_trial_run is True:
        outputdir_name += '_test'
        yieldstep = 0.1
        finaltime = 4
    else:
        yieldstep = 0.01
        finaltime = 31
    pro_instance = project.Project(['data','flumes','saman_2007'],
                                       outputdir_name=outputdir_name)
    mesh_filename = pro_instance.meshdir + basename + '.msh'

    #--------------------------------------------------------------------------
    # Copy scripts to output directory and capture screen
    # output to file
    #--------------------------------------------------------------------------

    # creates copy of code in output dir
    print "The output dir is", pro_instance.outputdir
    copy_code_files(pro_instance.outputdir,__file__,
                    dirname(project.__file__) \
                    + sep + project.__name__+'.py')
    copy (pro_instance.codedir + 'run_dam.py',
          pro_instance.outputdir + 'run_dam' + str(friction)+ '.py')
    copy (pro_instance.codedir + 'create_mesh.py',
          pro_instance.outputdir + 'create_mesh.py')
    copy (pro_instance.codedir + 'project.py',
          pro_instance.outputdir + 'project.py')
    if is_trial_run is False:
        start_screen_catcher(pro_instance.outputdir, int(friction*100))

    print 'USER:    ', pro_instance.user
    #-------------------------------------------------------------------------
    # Create the triangular mesh
    #-------------------------------------------------------------------------
    
    #beach_angle = 15.0
    #toe_to_nails_distance = .5
    create_mesh.generate(mesh_filename,
                         beach_angle,
                         toe_to_nails_distance,
                         is_coarse=is_trial_run) # this creates the mesh

    head,tail = path.split(mesh_filename)
    copy (mesh_filename,
          pro_instance.outputdir + tail )
    #-------------------------------------------------------------------------
    # Setup computational domain
    #-------------------------------------------------------------------------
    domain = Domain(mesh_filename, use_cache = False, verbose = True)
   

    print 'Number of triangles = ', len(domain)
    print 'The extent is ', domain.get_extent()
    print domain.statistics()

    
    domain.set_name(basename)
    domain.set_datadir(pro_instance.outputdir)
    domain.set_quantities_to_be_stored(['stage', 'xmomentum', 'ymomentum'])
    domain.set_minimum_storable_height(0.01)
    #domain.set_store_vertices_uniquely(True)  # for writting to sww


    def elevation_function(x,y):
        from Numeric import zeros, size, Float
        slope = create_mesh.xslope 
    
        z = zeros(size(x), Float)
        for i in range(len(x)):
            if x[i] < slope:
                z[i] = 0.0
            else:
                z[i] = (x[i]-slope)*tan(radians(beach_angle))
        return z

    #-------------------------------------------------------------------------
    # Setup initial conditions
    #-------------------------------------------------------------------------
        
    domain.set_quantity('stage', elevation_function)
    domain.set_quantity('friction', friciton) 
    domain.set_quantity('elevation', elevation_function)

    print 'Available boundary tags', domain.get_boundary_tags()
    domain.set_region('dam','stage',0.30,
                                 location = 'unique vertices') 

    Br = Reflective_boundary(domain)
    Bd = Dirichlet_boundary([0,0,0])  # to drain the water out.
    domain.set_boundary( {'wall': Br, 'edge': Bd} )

    #-------------------------------------------------------------------------
    # Evolve system through time
    #-------------------------------------------------------------------------
    t0 = time.time()

    for t in domain.evolve(yieldstep, finaltime):
        domain.write_time()
    
        print 'That took %.2f seconds' %(time.time()-t0)
        print 'finished'


    #-------------------------------------------------------------------------
    # Calculate gauge info
    #-------------------------------------------------------------------------
    if not is_trial_run:
        points = [[gate_position - 0.65,0.2],
                  [gate_position - 0.55,0.2],
                  [gate_position - 0.45,0.2],
                  [gate_position - 0.35,0.2],
                  [gate_position - 0.25,0.2]
                  ]
        interpolate_sww2csv(pro_instance.outputdir + basename +".sww",
                            points,
                            pro_instance.outputdir + \
                            "depth_manning_"+str(friction)+".csv",
                            pro_instance.outputdir + "velocity_x.csv",
                            pro_instance.outputdir + "velocity_y.csv")
  
    return pro_instance

#-------------------------------------------------------------
if __name__ == "__main__":
    beach_angle = 3.0
    toe_to_nails_distance = 0.1
    for friction in [0.0,0.01]:
        main(friction,beach_angle, toe_to_nails_distance,
             is_trial_run = True, outputdir_name='testing')
    
